@@ -32,7 +32,7 @@ G_BEGIN_DECLS
 TrackerResource *tracker_extract_new_artist (const char *name);
 TrackerResource *tracker_extract_new_contact (const char *fullname);
 TrackerResource *tracker_extract_new_equipment (const char *make, const char *model);
-TrackerResource *tracker_extract_new_external_reference (const char *source_uri, const char *identifier);
+TrackerResource *tracker_extract_new_external_reference (const char *source_uri, const char *identifier, const char *uri);
 TrackerResource *tracker_extract_new_location (const char *address, const char *state, const char *city, const char *country, const char *gps_altitude, const char *gps_latitude, const char *gps_longitude);
 TrackerResource *tracker_extract_new_music_album_disc (const char *album_title, TrackerResource *album_artist, int disc_number, const char *date);
 TrackerResource *tracker_extract_new_tag (const char *label);
