@@ -99,14 +99,14 @@ tracker_power_class_init (TrackerPowerClass *klass)
 	                                                       "Battery in use",
 	                                                       "Whether the battery is being used",
 	                                                       FALSE,
-	                                                       G_PARAM_READABLE));
+	                                                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (object_class,
 	                                 PROP_ON_LOW_BATTERY,
 	                                 g_param_spec_boolean ("on-low-battery",
 	                                                       "Battery low",
 	                                                       "Whether the battery is low",
 	                                                       FALSE,
-	                                                       G_PARAM_READABLE));
+	                                                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
