@@ -26,6 +26,8 @@
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
 
+#include <libtracker-sparql/tracker-sparql.h>
+
 G_BEGIN_DECLS
 
 typedef struct {
@@ -49,7 +51,8 @@ void        tracker_toc_add_entry       (TrackerToc *toc,
                                          gdouble     duration);
 
 TrackerToc *tracker_cue_sheet_parse     (const gchar *cue_sheet);
-TrackerToc *tracker_cue_sheet_guess_from_uri (const gchar *uri);
+TrackerToc *tracker_cue_sheet_guess_from_uri (TrackerSparqlConnection *conn,
+                                              const gchar             *uri);
 
 G_END_DECLS
 
