@@ -171,6 +171,7 @@ tracker_extract_get_metadata (TrackerExtractInfo  *extract_info,
                               GError             **error)
 {
 	TrackerResource *metadata;
+	TrackerConfig *config;
 	ODTMetadataParseInfo info = { 0 };
 	ODTFileType file_type;
 	GFile *file;
@@ -198,6 +199,9 @@ tracker_extract_get_metadata (TrackerExtractInfo  *extract_info,
 
 	uri = g_file_get_uri (file);
 
+	/* Setup conf */
+	config = tracker_main_get_config ();
+
 	g_debug ("Extracting OASIS metadata and contents from '%s'", uri);
 
 	/* First, parse metadata */
@@ -232,7 +236,7 @@ tracker_extract_get_metadata (TrackerExtractInfo  *extract_info,
 
 	/* Extract content with the given limitations */
 	extract_oasis_content (uri,
-	                       tracker_extract_info_get_max_text (extract_info),
+	                       tracker_config_get_max_bytes (config),
 	                       file_type,
 	                       metadata);
 
@@ -256,13 +260,13 @@ xml_start_element_handler_metadata (GMarkupParseContext  *context,
 {
 	ODTMetadataParseInfo *data = user_data;
 
-	#define push_tag(id)                                                             \
+	#define push_tag(id) \
 		g_queue_push_head (data->tag_stack, GINT_TO_POINTER(id));
 
-	#define handle_tag_and_return(name, id)                                          \
-		if (g_ascii_strcasecmp (element_name, name) == 0) {                          \
-			push_tag (id);                                                           \
-			return;                                                                  \
+	#define handle_tag_and_return(name, id) \
+		if (g_ascii_strcasecmp (element_name, name) == 0) { \
+			push_tag (id); \
+			return; \
 		};
 
 	handle_tag_and_return ("dc:title", ODT_TAG_TYPE_TITLE);
@@ -307,7 +311,7 @@ xml_start_element_handler_metadata (GMarkupParseContext  *context,
 	push_tag (ODT_TAG_TYPE_UNKNOWN);
 
 	#undef push_tag
-	#undef handle_tag
+	#undef handle_tag_and_return
 }
 
 static void
@@ -447,13 +451,19 @@ xml_start_element_handler_content (GMarkupParseContext  *context,
 {
 	ODTContentParseInfo *data = user_data;
 
-	#define push_tag(id)                                                             \
+	#define push_tag(id) \
 		g_queue_push_head (data->tag_stack, GINT_TO_POINTER(id));
 
-	#define handle_tag_and_return(name, id)                                          \
-		if (g_ascii_strcasecmp (element_name, name) == 0) {                          \
-			push_tag (id);                                                           \
-			return;                                                                  \
+	#define handle_tag_and_return(name, id) \
+		if (g_ascii_strcasecmp (element_name, name) == 0) { \
+			push_tag (id); \
+			return; \
+		};
+
+	#define handle_tag_and_return_n(name, id, n) \
+		if (g_ascii_strncasecmp (element_name, name, n) == 0) { \
+			push_tag (id); \
+			return; \
 		};
 
 	switch (data->file_type) {
@@ -475,12 +485,12 @@ xml_start_element_handler_content (GMarkupParseContext  *context,
 		return;
 
 	case FILE_TYPE_ODS:
-		handle_tag_and_return ("text", ODT_TAG_TYPE_SPREADSHEET_TEXT);
+		handle_tag_and_return_n ("text", ODT_TAG_TYPE_SPREADSHEET_TEXT, 4);
 		push_tag (ODT_TAG_TYPE_UNKNOWN);
 		return;
 
 	case FILE_TYPE_ODG:
-		handle_tag_and_return ("text", ODT_TAG_TYPE_GRAPHICS_TEXT);
+		handle_tag_and_return_n ("text", ODT_TAG_TYPE_GRAPHICS_TEXT, 4);
 		push_tag (ODT_TAG_TYPE_UNKNOWN);
 		return;
 
@@ -491,7 +501,8 @@ xml_start_element_handler_content (GMarkupParseContext  *context,
 	}
 
 	#undef push_tag
-	#undef handle_tag
+	#undef handle_tag_and_return
+	#undef handle_tag_and_return_n
 }
 
 static void
