@@ -285,12 +285,14 @@ G_MODULE_EXPORT gboolean
 tracker_extract_get_metadata (TrackerExtractInfo  *info,
                               GError             **error)
 {
+	TrackerConfig *config;
 	time_t creation_date;
 	GError *inner_error = NULL;
 	TrackerResource *metadata;
 	TrackerXmpData *xd = NULL;
 	PDFData pd = { 0 }; /* actual data */
 	PDFData md = { 0 }; /* for merging */
+	G_GNUC_UNUSED GBytes *bytes;
 	PopplerDocument *document;
 	gchar *xml = NULL;
 	gchar *content, *uri;
@@ -345,7 +347,13 @@ tracker_extract_get_metadata (TrackerExtractInfo  *info,
 	g_free (filename);
 	uri = g_file_get_uri (file);
 
+#if POPPLER_CHECK_VERSION (0, 82, 0)
+	bytes = g_bytes_new (contents, len);
+	document = poppler_document_new_from_bytes (bytes, NULL, &inner_error);
+	g_bytes_unref (bytes);
+#else
 	document = poppler_document_new_from_data (contents, len, NULL, &inner_error);
+#endif
 
 	if (inner_error) {
 		if (inner_error->code == POPPLER_ERROR_ENCRYPTED) {
@@ -544,7 +552,8 @@ tracker_extract_get_metadata (TrackerExtractInfo  *info,
 
 	tracker_resource_set_int64 (metadata, "nfo:pageCount", poppler_document_get_n_pages(document));
 
-	n_bytes = tracker_extract_info_get_max_text (info);
+	config = tracker_main_get_config ();
+	n_bytes = tracker_config_get_max_bytes (config);
 	content = extract_content_text (document, n_bytes);
 
 	if (content) {
