@@ -40,6 +40,29 @@
 #include "tracker-extract.h"
 #include "tracker-read.h"
 
+static gboolean
+allow_file (GSList      *text_allowlist_patterns,
+            GFile       *file)
+{
+	GSList *l;
+	g_autofree gchar *basename = NULL;
+
+	basename = g_file_get_basename (file);
+
+	for (l = text_allowlist_patterns; l; l = l->next) {
+#if GLIB_CHECK_VERSION (2, 70, 0)
+		if (g_pattern_spec_match_string (l->data, basename))
+#else
+		if (g_pattern_match_string (l->data, basename))
+#endif
+		{
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static gchar *
 get_file_content (GFile   *file,
                   gsize    n_bytes,
@@ -81,10 +104,14 @@ tracker_extract_get_metadata (TrackerExtractInfo  *info,
                               GError             **error)
 {
 	TrackerResource *metadata;
+	TrackerConfig *config;
 	GFile *file;
+	GSList *text_allowlist_patterns;
 	gchar *content = NULL, *resource_uri;
 	GError *inner_error = NULL;
 
+	config = tracker_main_get_config ();
+	text_allowlist_patterns = tracker_config_get_text_allowlist_patterns (config);
 	file = tracker_extract_info_get_file (info);
 
 	resource_uri = tracker_file_get_content_identifier (file, NULL, NULL);
@@ -92,21 +119,23 @@ tracker_extract_get_metadata (TrackerExtractInfo  *info,
 	tracker_resource_add_uri (metadata, "rdf:type", "nfo:PlainTextDocument");
 	g_free (resource_uri);
 
-	content = get_file_content (tracker_extract_info_get_file (info),
-	                            tracker_extract_info_get_max_text (info),
-	                            &inner_error);
+	if (allow_file (text_allowlist_patterns, file)) {
+		content = get_file_content (tracker_extract_info_get_file (info),
+		                            tracker_config_get_max_bytes (config),
+		                            &inner_error);
 
-	if (inner_error != NULL) {
-		/* An error occurred, perhaps the file was deleted. */
-		g_propagate_prefixed_error (error, inner_error, "Could not open:");
-		return FALSE;
-	}
+		if (inner_error != NULL) {
+			/* An error occurred, perhaps the file was deleted. */
+			g_propagate_prefixed_error (error, inner_error, "Could not open:");
+			return FALSE;
+		}
 
-	if (content) {
-		tracker_resource_set_string (metadata, "nie:plainTextContent", content);
-		g_free (content);
-	} else {
-		tracker_resource_set_string (metadata, "nie:plainTextContent", "");
+		if (content) {
+			tracker_resource_set_string (metadata, "nie:plainTextContent", content);
+			g_free (content);
+		} else {
+			tracker_resource_set_string (metadata, "nie:plainTextContent", "");
+		}
 	}
 
 	tracker_extract_info_set_resource (info, metadata);
